"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const graphql_1 = require("graphql");
const graphql_transformer_core_1 = require("graphql-transformer-core");
const graphql_transformer_common_1 = require("graphql-transformer-common");
class AutoTransformer extends graphql_transformer_core_1.Transformer {
    constructor() {
        super('AutoTransformer', graphql_transformer_core_1.gql `
        directive @auto(creatable: Boolean = false, updatable: Boolean = false) on FIELD_DEFINITION
      `);
        this.field = (parent, definition, directive, ctx) => {
            var _a;
            if (parent.kind === graphql_1.Kind.INTERFACE_TYPE_DEFINITION) {
                throw new graphql_transformer_core_1.InvalidDirectiveError(`The @auto directive cannot be placed on an interface's field. See ${parent.name.value}${definition.name.value}`);
            }
            const modelDirective = (_a = parent.directives) === null || _a === void 0 ? void 0 : _a.find((dir) => dir.name.value === 'model');
            if (!modelDirective) {
                throw new graphql_transformer_core_1.InvalidDirectiveError('Types annotated with @auto must also be annotated with @model.');
            }
            if (!graphql_transformer_common_1.isNonNullType(definition.type)) {
                throw new graphql_transformer_core_1.TransformerContractError(`@auto directive can only be used on non-nullable type fields`);
            }
            const isArg = (s) => (arg) => arg.name.value === s;
            const getArg = (arg, dflt) => {
                var _a;
                const argument = (_a = directive.arguments) === null || _a === void 0 ? void 0 : _a.find(isArg(arg));
                return argument ? graphql_1.valueFromASTUntyped(argument.value) : dflt;
            };
            const creatable = getArg('creatable', false);
            const updatable = getArg('updatable', false);
            this.updateCreateInput(ctx, parent.name.value, definition, creatable);
            this.updateUpdateInput(ctx, parent.name.value, definition, updatable);
        };
    }
    updateCreateInput(ctx, typeName, autoField, nullable) {
        this.updateInput(ctx, graphql_transformer_common_1.ModelResourceIDs.ModelCreateInputObjectName(typeName), typeName, autoField, nullable);
    }
    updateUpdateInput(ctx, typeName, autoField, nullable) {
        this.updateInput(ctx, graphql_transformer_common_1.ModelResourceIDs.ModelUpdateInputObjectName(typeName), typeName, autoField, nullable);
    }
    updateInput(ctx, inputName, typeName, autoField, nullable) {
        const input = ctx.getType(inputName);
        if (input && input.kind === graphql_1.Kind.INPUT_OBJECT_TYPE_DEFINITION) {
            if (input.fields) {
                if (nullable) {
                    // make autoField nullable
                    ctx.putType({
                        ...input,
                        fields: input.fields.map((f) => {
                            if (f.name.value === autoField.name.value) {
                                return graphql_transformer_common_1.makeInputValueDefinition(autoField.name.value, graphql_transformer_common_1.unwrapNonNull(autoField.type));
                            }
                            return f;
                        }),
                    });
                }
                else {
                    // or strip autoField
                    const updatedFields = input.fields.filter((f) => f.name.value !== autoField.name.value);
                    if (updatedFields.length === 0) {
                        throw new graphql_transformer_core_1.InvalidDirectiveError(`After stripping away version field "${autoField.name.value}", \
                        the create input for type "${typeName}" cannot be created \
                        with 0 fields. Add another field to type "${typeName}" to continue.`);
                    }
                    ctx.putType({
                        ...input,
                        fields: updatedFields,
                    });
                }
            }
        }
    }
}
exports.AutoTransformer = AutoTransformer;
//# sourceMappingURL=AutoTransformer.js.map